/*
 * Jalview - A Sequence Alignment Editor and Viewer (Version 2.8.2)
 * Copyright (C) 2014 The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.io;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import jalview.analysis.SecStrConsensus.SimpleBP;
import jalview.datamodel.AlignmentAnnotation;
import jalview.datamodel.Annotation;
import jalview.datamodel.Sequence;
import jalview.datamodel.SequenceI;

import fr.orsay.lri.varna.exceptions.ExceptionFileFormatOrSyntax;
import fr.orsay.lri.varna.exceptions.ExceptionLoadingFailed;
import fr.orsay.lri.varna.exceptions.ExceptionPermissionDenied;
import fr.orsay.lri.varna.factories.RNAFactory;
import fr.orsay.lri.varna.models.rna.RNA;

public class RnamlFile extends AlignFile
{
  public String id;

  protected ArrayList<RNA> result;

  public RnamlFile()
  {
    super();

  }

  public RnamlFile(String inFile, String type) throws IOException
  {
    super(inFile, type);

  }

  public RnamlFile(FileParse source) throws IOException
  {
    super(source);

  }

  public BufferedReader CreateReader() throws FileNotFoundException
  {
    FileReader fr = null;
    fr = new FileReader(inFile);

    BufferedReader r = new BufferedReader(fr);
    return r;
  }

  /*
   * (non-Javadoc)
   * 
   * @see jalview.io.AlignFile#parse()
   */
  public void parse() throws IOException
  {
    if (System.getProperty("java.version").indexOf("1.6") > -1
            || System.getProperty("java.version").indexOf("1.5") > -1)
    {
      // patch for 'This parser does not support specification "null" version
      // "null"' error
      // this hack ensures we get a properly updated SAXParserFactory on older
      // JVMs
      // thanks to Stefan Birkner over at https://coderwall.com/p/kqsrrw
      System.setProperty("javax.xml.parsers.SAXParserFactory",
              "com.sun.org.apache.xerces.internal.jaxp.SAXParserFactoryImpl");
    }
    // rather than lose exception semantics whilst parsing RNAML with VARNA we
    // wrap the routine and catch all exceptions before passing them up the
    // chain as an IOException
    try
    {
      _parse();
    } catch (ExceptionPermissionDenied pdx)
    {
      errormessage = "Couldn't access datasource (" + pdx.getMessage()
              + ")";
      throw new IOException(pdx);
    } catch (ExceptionLoadingFailed lf)
    {
      errormessage = "Couldn't process data as RNAML file ("
              + lf.getMessage() + ")";
      throw new IOException(lf);
    } catch (ExceptionFileFormatOrSyntax iff)
    {
      errormessage = "Invalid RNAML file (" + iff.getMessage() + ")";
      throw new IOException(iff);
    } catch (Exception x)
    {
      error = true;
      errormessage = "Problem parsing data as RNAML (" + x.getMessage()
              + ")";
      throw new IOException("Couldn't parse the datasource as RNAML", x);
    }
  }

  @SuppressWarnings("unchecked")
  public void _parse() throws FileNotFoundException,
          ExceptionPermissionDenied, ExceptionLoadingFailed,
          ExceptionFileFormatOrSyntax
  {

    result = RNAFactory.loadSecStrRNAML(getReader());

    ArrayList<ArrayList> allarray = new ArrayList();
    ArrayList<ArrayList<SimpleBP>> BP = new ArrayList();
    ArrayList strucinarray = new ArrayList();
    SequenceI[] seqs = new SequenceI[result.size()];

    for (int i = 0; i < result.size(); i++)
    {

      RNA current = result.get(i);
      String rna = current.getStructDBN(true);
      String seq = current.getSeq();
      int begin = 1;
      int end = seq.length();

      id = current.getName();
      seqs[i] = new Sequence(id, seq, begin, end);

      seqs[i].setEnd(seqs[i].findPosition(seqs[i].getLength()));
      String[] annot = new String[rna.length()];
      Annotation[] ann = new Annotation[rna.length()];

      for (int j = 0; j < rna.length(); j++)
      {
        annot[j] = "" + rna.charAt(j);

      }
      for (int k = 0; k < rna.length(); k++)
      {
        ann[k] = new Annotation(annot[k], "",
                jalview.schemes.ResidueProperties.getRNASecStrucState(
                        annot[k]).charAt(0), 0f);
      }

      AlignmentAnnotation align = new AlignmentAnnotation("Sec. str.",
              current.getID(), ann);

      seqs[i].addAlignmentAnnotation(align);
      seqs[i].setRNA(result.get(i));

      allarray.add(strucinarray);

      annotations.addElement(align);
      BP.add(align.bps);

    }

    setSeqs(seqs);
  }

  public static String print(SequenceI[] s)
  {
    return "not yet implemented";
  }

  public String print()
  {
    System.out.print("print :");
    return print(getSeqsAsArray());
  }

  public ArrayList getRNA()
  {
    return result;
  }

  // public static void main(String[] args) {
  // Pattern p= Pattern.compile("(.+)[.][^.]+");
  // Matcher m = p.matcher("toto.xml.zip");
  // System.out.println(m.matches());
  // System.out.println(m.group(1));
  // }
  /**
   * make a friendly ID string.
   * 
   * @param dataName
   * @return truncated dataName to after last '/'
   */
  private String safeName(String dataName)
  {
    int b = 0;
    while ((b = dataName.indexOf("/")) > -1 && b < dataName.length())
    {
      dataName = dataName.substring(b + 1).trim();

    }
    int e = (dataName.length() - dataName.indexOf(".")) + 1;
    dataName = dataName.substring(1, e).trim();
    return dataName;
  }
}
